SCADA Errata Memo
--------------
This memo details the shortcomings and errors of the 2016 software team.  I believe we provided a robust data aquisition system, and not much else.

### The software is not performant.
 While the software appears to be able to keep up with the traffic on a desktop processor.  When switching to the raspberry pi this quickly becomes an issue.  It can only process small amounts of traffic.

 When processing full force CAN traffic we see 100% usage on one of the cores. There are two possible explanations:
 1. We made a severe implementation error.  Poor understanding of the libraries or language could be to blame.
 2. The work load is CPU bound in nature.  I initially assumed it would be I/O bound.  Python is a poor choice because it lacks parallelism by design. A systems programming language would be better suited for our performance needs.

If 1, then profiling the software should give some insight.  If 2, then the software should be rewritten in a systems programming language like C, Java, or Go.  I would stay away from C, and prefer Go over Java.  Rewriting the software is **not** the same thing as redesigning it.  I believe the architecture is correct in modeling the problem and it does not need to be changed significantly.  It's only 1500 lines, it could be reimplemented in a few weeks.  You could likely achieve the same functionality in Go using only the standard libraries.  Java and C don't even have JSON in the standard libraries, so good luck with those.

As a feasibility test I would try to get just the data modeling and logging implemented.  And do some benchmarks.  If it works, finish up the implementation, if not, then I'm totally wrong and you should ignore me.

NOTE: The compile once, run anywhere requirement is silly. Compiling a statically linked binary would greatly simplify deployment.

### The Message Bus Protocol is Sloppy.
The message bus protocol is basically to send out a full system and then diffs of significant objects and their location in the system tree.  You can also update an object by sending a partial section of the tree.  The attributes described will be copied recursively, leaving unmentioned attributes untouched. There are two ways to do the same thing.  It may not be a major issue but I really don't like the way it turned out.

### The software does not meet the requirements.
In this section I will enumerate the requirements that were not met and explain how they could be solved using the existing architecture

##### Demo Mode
A virtual subsystem called DemoSubsystem could be created to inject pretend systems onto the CAN Bus or UNIX socket (to emulate a serial port).  It could alternatively attach to the physical subsystems and give them pretend messages directly. This system presents options to the user to turn on or off various subsystems.  I never considered what it would be like to have a virtual subsystem create other virtual subsystems, but that is also an interesting idea.

Then you monitor the virtual subsystem and it's physical counterpart to make sure they behave the same and tweak as necessary.  This could produce really good models.  If anyone on MCM could code...

##### Scripting
The controllables need to have some kind of locking mechanism to signify who has control over them.  A controllable can be locked and owned by a virtual subsystem.  The user would be informed that this is the case, and would have to tell the virtual subsystem to release control.  The virtual subsystem could present scripting options.  Maybe an in browser text editor, and the contents are sent to the server for execution.

NOTE: This should be the limit of the control that the linux machine provides.  It should not do cruise control, it should not do launch control.  It shouldn't implement anti-lock brakes. Those sorts of control systems should be done by something with hard real time capabilities.

##### Calibration
Measurands should have an adjustable cal factor present in their view.  It gets sent back to the server, it's the only part of a measurand the user can control.  Simple.  We should have done this, but we didn't. The system should provide arbitrary key value storage to the subsystems so they can store this stuff, but that was completely overlooked.

### Web Client API
The web client API works well for the physical subsystems, but it is severely underpowered and insufficient to handle the extensibility that the virtual subsystems bring to the mix.  Should each virtual subsystem expose it's own API? and remote method call from the web be routed to them? I don't have any answers here.  Someone needs to think about it more.
